# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'OneTimeCallback.is_view'
        db.add_column('onetime_onetimecallback', 'is_view',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)

    def backwards(self, orm):
        # Deleting field 'OneTimeCallback.is_view'
        db.delete_column('onetime_onetimecallback', 'is_view')

    models = {
        'onetime.onetimecallback': {
            'Meta': {'object_name': 'OneTimeCallback'},
            'callback': ('onetime.fields.CallableField', [], {}),
            'context': ('onetime.fields.PythonLiteralField', [], {}),
            'id': ('onetime.fields.UUIDField', [], {'max_length': '64', 'primary_key': 'True'}),
            'is_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['onetime']


from django.conf import settings

from onetime.models import OneTimeCallback


class CallbackMiddleware(object):
    def process_request(self, request):
        request_param = getattr(settings, 'ONETIME_REQUEST_PARAMETER', 'onetime_id')
        onetime_id = request.REQUEST.get(request_param, None)
        if onetime_id is None:
            return
        try:
            entry = OneTimeCallback.objects.get(id=onetime_id)
        except OneTimeCallback.DoesNotExist:
            return

        args, kwargs = entry.context
        if entry.is_view:
            retval = entry.callback(request, *args, **kwargs)
        else:
            retval = entry.callback(*args, **kwargs)
        entry.delete()
        return retval



import ast
import sys
import importlib
import uuid

from django.db import models


class UUIDField(models.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 64)
        kwargs['blank'] = True
        models.CharField.__init__(self, *args, **kwargs)

    def pre_save(self, model_instance, is_new):
        if is_new:
            value = str(uuid.uuid4())
            setattr(model_instance, self.attname, value)
            return value
        else:
            return super(models.CharField, self).pre_save(model_instance, is_new)


class CallableField(models.TextField):
    __metaclass__ = models.SubfieldBase

    def get_prep_value(self, value):
        """
        Generate the string representation of the callable.
        """
        module_name = value.__module__
        for k, v in sys.modules[module_name].__dict__.iteritems():
            if v is value:
                callable_name = k
                break
        else:
            callable_name = getattr(value, 'func_name', None)
            if callable_name is None:
                callable_name = getattr(value, '__name__', None)
                if callable_name is None:
                    raise ValueError("Cannot determine callable name for object %r" % value)
        return '.'.join((module_name, callable_name))

    def to_python(self, value):
        """
        Retrieve the callable object from the string value.
        """
        if callable(value):
            return value

        if not value:
            return value

        module_name, callable_name = value.rsplit('.', 1)
        module = sys.modules.get(module_name)
        if module is None:
            module = importlib.import_module(module_name)
            # raise ImportError("Cannot find module %s in sys.modules" % module_name)
        callback = getattr(module, callable_name, None)
        if callback is None:
            raise ImportError("Cannot find callable %s in module %s" % (callable_name, module_name))
        if not callable(callback):
            raise TypeError("%r is not callable." % callback)
        return callback


class PythonLiteralField(models.TextField):
    """
    Stores basic Python primitives to the DB.

    Can store numbers, strings, booleans, tuples, lists, dicts.
    """
    __metaclass__ = models.SubfieldBase

    def get_prep_value(self, value):
        return repr(value)

    def to_python(self, value):
        try:
            return ast.literal_eval(value)
        except (SyntaxError, ValueError):
            return value
            #  raise ValueError("Stored literals cannot contain complex types.  Could not evaluate: %r" % (value,))

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], [
        "^onetime\.fields\.UUIDField",
        "^onetime\.fields\.CallableField",
        "^onetime\.fields\.PythonLiteralField"
    ])
except ImportError:
    pass



from __future__ import absolute_import


def call_later(callback, *args, **kwargs):
    otc = models.OneTimeCallback.objects.create(callback=callback, context=(args, kwargs))
    return otc.id


def call_view_later(callback, *args, **kwargs):
    otc = models.OneTimeCallback.objects.create(callback=callback, context=(args, kwargs), is_view=True)
    return otc.id

from . import models
from . import fields
from . import middleware



from django.db import models

from onetime import fields


class OneTimeCallback(models.Model):
    id = fields.UUIDField(primary_key=True, editable=False)
    callback = fields.CallableField()
    context = fields.PythonLiteralField()
    is_view = models.BooleanField(default=False)

    def __unicode__(self):
        return "<OneTimeCallback: %s(%s)" % (self.callback.func_name, ','.join(list(map(str, self.context[0])) + ['%s=%r' % (k, v) for k, v in self.context[1].items()]))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'OneTimeCallback'
        db.create_table('onetime_onetimecallback', (
            ('id', self.gf('onetime.fields.UUIDField')(max_length=64, primary_key=True)),
            ('callback', self.gf('onetime.fields.CallableField')()),
            ('context', self.gf('onetime.fields.PythonLiteralField')()),
        ))
        db.send_create_signal('onetime', ['OneTimeCallback'])

    def backwards(self, orm):
        # Deleting model 'OneTimeCallback'
        db.delete_table('onetime_onetimecallback')

    models = {
        'onetime.onetimecallback': {
            'Meta': {'object_name': 'OneTimeCallback'},
            'callback': ('onetime.fields.CallableField', [], {}),
            'context': ('onetime.fields.PythonLiteralField', [], {}),
            'id': ('onetime.fields.UUIDField', [], {'max_length': '64', 'primary_key': 'True'})
        }
    }

    complete_apps = ['onetime']


from distutils.core import setup
from setuptools import find_packages

try:
    import importlib
    required_libs = ()
except ImportError:
    required_libs = ('importlib',)


setup(
    name='django-onetime',
    version='0.1.6',
    author=u'Don Spaulding',
    author_email='don@mirusresearch.com',
    packages=find_packages(),
    install_requires=required_libs,
    url='http://bitbucket.org/mirusresearch/django-onetime',
    license='MIT license, see LICENSE.txt',
    description='A reusable Django app that lets you call' + \
                ' followup functions after an event has taken place.' + \
                '  (Such as two-factor authentication)',
    long_description=open('README.rst').read(),
    zip_safe=False,
    include_package_data=True
)
